@@ -182,4 +182,49 @@ mod tests {
             }
         )
     }
+
+    pub fn multiple_generics_test_case() -> Ast {
+        parse_quote!(
+            #[builder]
+            impl<T> Request<T> {
+                pub fn fake_new<K, V>(
+                    headers: Vec<(K, V)>,
+                    uri: Option<http::Uri>,
+                    method: Option<http::Method>,
+                    body: T,
+                ) -> http::Result<Request<T>>
+                where
+                    HeaderName: TryFrom<K>,
+                    <HeaderName as TryFrom<K>>::Error: Into<http::Error>,
+                    HeaderValue: TryFrom<V>,
+                    <HeaderValue as TryFrom<V>>::Error: Into<http::Error>,
+                {
+                    let mut builder = http::request::Builder::new();
+                    for (key, value) in headers {
+                        builder = builder.header(key, value);
+                    }
+                    let req = builder.body(body)?;
+
+                    Ok(Self { inner: req })
+                }
+            }
+        )
+    }
+
+    pub fn collections_generics_test_case2() -> Ast {
+        parse_quote!(
+            #[builder]
+            impl Collections {
+                fn new<K: Into<String> + Eq + Hash, V: Into<String>>(
+                    map: HashMap<K, V>,
+                    set: HashSet<K>,
+                ) -> Collections {
+                    Self {
+                        map: map.into_iter().map(|(k, v)| (k.into(), v.into())).collect(),
+                        set: set.into_iter().map(|v| v.into()).collect(),
+                    }
+                }
+            }
+        )
+    }
 }
